__all__ = [
    "eff",
    "snli",
    "squad",
    "reddit",
    "cityscapes",
    "nlp_progress",
    "coqa",
    "chexpert",
    "cmrc",
]


from sota_extractor.scrapers.eff import eff
from sota_extractor.scrapers.snli import snli
from sota_extractor.scrapers.squad import squad
from sota_extractor.scrapers.reddit import reddit
from sota_extractor.scrapers.cityscapes import cityscapes
from sota_extractor.scrapers.nlp_progress import nlp_progress
from sota_extractor.scrapers.coqa import coqa
from sota_extractor.scrapers.chexpert import chexpert
from sota_extractor.scrapers.cmrc import cmrc
